$(document).ready(function() {
    add_modal_to_document();
    $('#div_modal').modal('hide');    // initialize modal instance to set options anywhere

    // to handle modal answer by event
    $('#div_modal .btn-ok').on('click', function() {
        set_modal_answer('ok');
        $(this).data('close') ? $('#div_modal').modal('hide') : trigger_modal_answer();
    });
    $('#div_modal .btn-cancel').on('click', function() {
        set_modal_answer('cancel');
        $(this).data('close') ? $('#div_modal').modal('hide') : trigger_modal_answer();
    });
    $('#div_modal .close').on('click', function() {
        set_modal_answer('cancel');
        // No need to close. If close x button clicked, alway close modal.
    });

    // When modal closes,
    // 1. trigger modal answer event
    // 2. call modal_callback
    // 3. unbind modal answer handler
    $('#div_modal').on('hidden.bs.modal', function() {
        trigger_modal_answer();
        if (typeof modal_callback === 'function') {
            console.log('. hidden.bs.modal[callback] :', modal_callback);
            modal_callback();
            modal_callback = null;
        }
        unbind_answer_handler();
    })

    // for closing modal with enter key
    $('#div_modal').on('shown.bs.modal', function() {
        $('#div_modal .btn-ok').focus();
    });

    // remove scrollbar's effect on body width
    $('.modal').on('show.bs.modal', function() {
        scrollbar_width = (window.innerWidth - $(document).width()) + 'px';
        $('body').css('margin-right', scrollbar_width);
    });
    $('.modal').on('hidden.bs.modal', function() {
        $('body').css('margin-right', '0px');
    });
});

// ---------------------------------------------------------------------

function trigger_modal_answer() {
    if (get_modal_answer()) {
        var event = 'modal.answer.' + get_modal_answer();
        console.log('. trigger_modal_answer() :', event);
        $(document).trigger(event);
        set_modal_answer('');
    }
}

function unbind_answer_handler() {
    console.log('. unbind_answer_handler()');
    $(document).off('modal.answer.ok');
    $(document).off('modal.answer.cancel');
}

// Should use unbind_answer_handler() to clear pre-set event handler.
function bind_answer_handler(answer, handler) {
    console.log('. bind_answer_handler():', answer, typeof handler);
    $(document).on('modal.answer.' + answer, handler);
}

// ---------------------------------------------------------------------

// to show no icon, use 'normal' type
var modal_callback;
function show_modal(title, msg, callback, mode) {
    console.log('- show_modal() :', title, '[' + msg.substr(0, 25) + '...]' , typeof callback, mode);
    mode = (typeof mode === 'undefined') ? 'normal' : mode;
    $('#div_modal').removeClass('modal-fti-normal modal-fti-confirm modal-fti-error modal-fti-warning')
                   .addClass('modal-fti-' + mode);
    $('#div_modal .modal-title').html(title);
    $('#div_modal .modal-body-msg').html(msg);
    $('#div_modal').modal('show');
    if (typeof callback === 'function') {
        modal_callback = callback;
    }
}

function hide_modal() {
    $('#div_modal').modal('hide');
}

function is_modal_shown() {
    return $('#div_modal').data('bs.modal').isShown;
}

// ---------------------------------------------------------------------
// Verizon style modal

function set_modal_mode(alert_confirm) {
    set_modal_button_enable('.btn-ok', true);
    set_modal_button_enable('.btn-cancel', (alert_confirm === 'confirm'));
}

function show_error_modal(title, msg, callback) {
    set_modal_default();
    set_modal_mode('alert');
    show_modal(title, msg, callback, 'error');
}

function show_notice_modal(title, msg, callback) {
    set_modal_default();
    set_modal_mode('alert');
    show_modal(title, msg, callback, 'confirm');
}

// to use confirm modal, add eventhandler on modal.answer.ok / .cancel
// should unbind_answer_handler() before add handler.
function show_confirm_modal(title, msg, callback) {
    set_confirm_cancel_modal();
    set_modal_mode('confirm');
    show_modal(title, msg, callback, 'confirm');
}

// ---------------------------------------------------------------------

function set_modal_button(selector, text, close) {
    var $btn = $('#div_modal ' + selector);
    $btn.text(text);
    $btn.data('close', close);
}

function set_modal_button_enable(selector, enable) {
    var $btn = $('#div_modal ' + selector);
    enable ? $btn.show() : $btn.hide();
}

function set_modal_footer_enable(enable) {
    var $footer = $('#div_modal .modal-footer');
    enable ? $footer.show() : $footer.hide();
}

function set_modal_close_x(enabled) {
    var text = enabled ? '×' : '';
    $('#div_modal button.close').text(text);
}

function set_modal_content(title, msg) {
    console.log('- set_modal_content() :', title, '[' + msg.substr(0, 25) + '...]');
    $('#div_modal .modal-title').html(title);
    $('#div_modal .modal-body-msg').html(msg);
}

function set_modal_backdrop(backdrop) {
    $('#div_modal').data('bs.modal').options.backdrop = backdrop;
}

// ---------------------------------------------------------------------
// unbind_answer_handler() is called implicitly by calling set_*_modal()

function set_modal_default() {
    set_ok_cancel_modal();
}

function set_ok_modal() {
    unbind_answer_handler();
    set_modal_unlock();
    set_modal_button_enable('.btn-ok', true);
    set_modal_button('.btn-ok', 'OK', true);
    set_modal_button_enable('.btn-cancel', false);
}

function set_ok_cancel_modal() {
    unbind_answer_handler();
    set_modal_unlock();
    set_modal_button_enable('.btn-ok', true);
    set_modal_button('.btn-ok', 'OK', true);
    set_modal_button_enable('.btn-cancel', true);
    set_modal_button('.btn-cancel', 'Cancel', true);
}

function set_yes_no_modal() {
    unbind_answer_handler();
    set_modal_unlock();
    set_modal_button_enable('.btn-ok', true);
    set_modal_button('.btn-ok', 'Yes', true);
    set_modal_button_enable('.btn-cancel', true);
    set_modal_button('.btn-cancel', 'No', true);
}

function set_confirm_cancel_modal() {
    unbind_answer_handler();
    set_modal_unlock();
    set_modal_button_enable('.btn-ok', true);
    set_modal_button('.btn-ok', 'Confirm', true);
    set_modal_button_enable('.btn-cancel', true);
    set_modal_button('.btn-cancel', 'Cancel', true);
}

function set_modal_lock() {
    // default backdrop option is 'static', but explict call for certainly
    set_modal_backdrop('static');
    set_modal_footer_enable(false);
    set_modal_close_x(false);
}

function set_modal_unlock() {
    set_modal_footer_enable(true);
    set_modal_close_x(true);
}

// ---------------------------------------------------------------------

var modal_answer;     // 'ok' / 'cancel'
function set_modal_answer(result) {
    modal_answer = result;
}

function get_modal_answer() {
    return modal_answer;
}

// ---------------------------------------------------------------------

function add_modal_to_document() {
    $(document.body).append('<div id="div_modal" class="modal fade" data-backdrop="static">'
        + '  <div class="modal-dialog">'
        + '    <div class="modal-content" >'
        + '      <div class="modal-header">'
        + '        <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>'
        + '        <h4 class="modal-title">&nbsp;</h4>'
        + '      </div>'
        + '      <div class="modal-body">'
        + '        <div class="modal-body-msg">&nbsp;</div>'
        + '      </div>'
        + '      <div class="modal-footer">'
        + '        <button type="button" class="btn btn-danger btn-ok" data-close="true">OK</button>'
        + '        <button type="button" class="btn btn-default btn-cancel" data-close="true">Cancel</button>'
        + '      </div>'
        + '    </div>'
        + '  </div>'
        + '</div>'
    );
}

// ---------------------------------------------------------------------
// Rebooting Notice Modal

// to use
// 1. add div_loading_modal html code
// 2. add start_wait_reboot_done as event handler
// ex) $(document).on('modal.answer.ok', start_wait_reboot_done);
// or  bind_answer_handler('ok', start_wait_reboot_done);

function start_wait_reboot_done() {
    console.log('- start_wait_reboot_done()');
    $('#div_rebooting_modal').modal('show');
    setTimeout(
        function() {
            $('#div_modal').off('hidden.bs.modal');
            $('#div_modal').on('hidden.bs.modal', function() {
                go_main_page();
            });
            wait_reboot_done({ Response: '' });
        },
        10 * 1000
    );
}

function wait_reboot_done(data) {
    console.log('- wait_reboot_done()', data);
    if (data['Response'] === 'NotSignedIn' || data['Response'] === 'Success') {
        $('#div_rebooting_modal').modal('hide');
        show_notice_modal('Rebooting Is Done', 'Rebooting is done. Now, you will be redirected to the main page.');
    } else {
        var cgi = '/vzw/cgi-bin/sign_in.cgi',
            req = { Command: "Status" };
        $.ajax({
            type: "POST",
            dataType: "json",
            timeout: 5000,
            url: cgi,
            data: JSON.stringify(req),
            success: wait_reboot_done,
            error: function (jqxhr, status, err) {
                wait_reboot_done({ Response: 'ajax error by timeout' });
            }
        });
    }
}

function add_rebooting_modal_to_document() {
    $(document.body).append(
        '<div id="div_rebooting_modal" class="modal fade" tabindex="-1" role="dialog" data-backdrop="static">'
        + '  <div class="modal-dialog">'
        + '    <div class="modal-content" >'
        + '      <div class="modal-header">'
        + '        <h4 class="modal-title">This Jetpack Will Now Restart</h4>'
        + '      </div>'
        + '      <div class="modal-body">'
        + '        Please wait. Connection to the Jetpack will automatically be re-established.'
        + '        <p class="text-center"><img src="/images/spinner.gif"></p>'
        + '      </div>'
        + '    </div>'
        + '  </div>'
        + '</div>'
    );
}

function add_rebooting_modal_to_document_for_factory_reset() {
    $(document.body).append(
        '<div id="div_rebooting_modal" class="modal fade" tabindex="-1" role="dialog" data-backdrop="static">'
        + '  <div class="modal-dialog">'
        + '    <div class="modal-content" >'
        + '      <div class="modal-header">'
        + '        <h4 class="modal-title">Restarting your Ellipsis Jetpack</h4>'
        + '      </div>'
        + '      <div class="modal-body">'
        + '        You have been Signed out and the Jetpack will now restart. Please reconnect to the Jetpack with the default Wi-Fi Name and Password on the Jetapack display.'
        + '        <p class="text-center"><img src="/images/spinner.gif"></p>'
        + '      </div>'
        + '    </div>'
        + '  </div>'
        + '</div>'
    );
	
}
