function request_pin_status() {
    console.log('- request_pin_status()');
    var cgi = '/vzw/cgi-bin/settings_sim.cgi',
        req = { Command : 'PinStatus' };
    get_json_by_post(
        cgi,
        JSON.stringify(req),
        check_pin_status_response
    );
}

function check_pin_status_response(data) {
    console.log('- check_pin_status_response() :', data);
    if (data['PinStatus'] === 'EnabledNotVerified') {
        show_sim_unlock_modal(data['PinRetries'], data['PukRetries']);
    } else if (data['PinStatus'] === 'Blocked') {
        show_sim_puk_modal(data['PukRetries']);
    } else if (data['PinStatus'] === 'PermanentlyBlocked') {
        show_sim_permanently_blocked_modal();
    } else {
        console.warn('w check_pin_status_response() : Unexpected PinStatus -', data['PinStatus']);
    }
}

//----------------------------------------------------------------------

function show_sim_unlock_modal(retries, puk_retries) {
    console.log('- show_sim_unlock_modal() :', retries, puk_retries);
    var title, msg, msg1, msg2, remain;
    if (retries === 0) {
        show_sim_puk_modal(puk_retries);
        return;
    }

    $('#div_modal').on('shown.bs.modal', function() {
        $('#OldPIN').focus();
    })

    // set modal style, title and msg
    set_ok_modal();
    set_modal_close_x(false);
    set_modal_button('.btn-ok', 'Unlock', false);
    title = 'SIM is PIN Locked';
    msg = $('#sim_modal_msg').html();
    $('#sim_modal_msg').remove();
    show_modal(title, msg);
    $('#OldPIN').focus();

    // set msg detail
    $('.sim_alert').html('');
    msg1 = 'You have locked the SIM card inside your Jetpack. ';
         + 'The PIN code must be entered correctly before the Jetpack will connect to the internet.';
    remain = (retries === 1) ? (retries + ' attempt remains') : (retries + ' attempts remain');
    msg2 = remain + ' until your SIM is permanently locked.';
    change_sim_modal_msg(msg1, msg2);

    // set_ok_modal() calls unbind_answer_handler();
    bind_answer_handler('ok', request_sim_unlock);
    $('#form-simunlock').on('submit', request_sim_unlock);
}

function request_sim_unlock() {
    console.log('- request_sim_unlock()', get_value('#OldPIN'));
    if (! get_value('#OldPIN')) {
        $('.sim_alert').html('Please enter your PIN');
        $('#OldPIN').focus().select();
        return false; // for stopping form submit
    }
    var cgi = '/vzw/cgi-bin/settings_sim.cgi',
        req = { Command : 'PinVerify', OldPIN : get_value('#OldPIN') };
    get_json_by_post(
        cgi,
        JSON.stringify(req),
        check_sim_unlock_response
    );
    set_form_loading('Checking PIN...');
    return false; // for stopping form submit
}

function check_sim_unlock_response(data) {
    console.log('- check_sim_unlock_response()', data);
    var title, msg, retries, remain, msg1, msg2;
    if (data['Response'] === 'Success') {
        console.log('. check_sim_unlock_response() : Success');
        $('[data-status=SignalStrength]').prop('src', '/images/transparent_1x1.png');
        go_main_page();
    } else if (data['Response'] === 'IncorrectPassword') {
        console.log('. check_sim_unlock_response() : IncorrectPassword - Retries:', data['PinRetries']);
        if (data['PinRetries'] === 0) {
            show_sim_puk_modal(data['PukRetries']);
        } else {
            set_form_loading_reset();
            msg1 = 'The PIN you have entered is not correct. Please verify the PIN code and try again.';
            retries = data['PinRetries'];
            remain = (retries === 1) ? (retries + ' attempt remains') : (retries + ' attempts remain');
            msg2 = remain + ' until your SIM is permanently locked.';
            change_sim_modal_msg(msg1, msg2);
            $('#OldPIN').focus().select();
        }
    }
}

//----------------------------------------------------------------------

function show_sim_puk_modal(retries) {
    console.log('- show_sim_puk_modal() :', retries);
    var title, msg, msg1, msg2, remain;
    if (retries === 0) {
        show_sim_permanently_blocked_modal();
        return;
    }

    $('#div_modal').on('shown.bs.modal', function() {
        $('#PUK').focus();
    })

    set_form_loading_reset();
    // set modal style, title and msg
    set_ok_modal();
    set_modal_button('.btn-ok', 'OK', false);
    title = 'SIM is PUK Locked';
    msg = $('#sim_puk_modal_msg').html();
    $('#sim_puk_modal_msg').remove();
    show_modal(title, msg);
    $('#PUK').focus();

    // set msg detail
    $('.sim_alert').html('');
    msg1 = 'You have entered incorrect SIM PIN too many times. '
         + 'Your SIM is permanently locked. Please contact Verizon customer support to unlock the SIM.';
    remain = (retries === 1) ? (retries + ' attempt remains') : (retries + ' attempts remain');
    msg2 = remain + ' until your SIM is permanently unusable.';
    change_sim_modal_msg(msg1, msg2);

    // set_ok_modal() calls unbind_answer_handler();
    bind_answer_handler('ok', request_sim_puk);
    $('#form-simpuk').on('submit', request_sim_puk);
}

function request_sim_puk() {
    console.log('- request_sim_puk()', get_value('#PUK'));
    var puk = $.trim(get_value('#PUK'));
    if (! puk || puk.length !== 8) {
        var puk_error_msg = puk ? 'PUK should be 8 characters.' : 'Please enter your PUK.';
        $('.sim_alert').html(puk_error_msg);
        $('#PUK').focus().select();
        return false; // for stopping form submit
    }
    var cgi = '/vzw/cgi-bin/settings_sim.cgi',
        req = { Command : 'PinUnblock', OldPIN : puk, NewPIN : '1111' };
    get_json_by_post(
        cgi,
        JSON.stringify(req),
        check_sim_puk_response
    );
    set_form_loading('Checking PUK...');
    return false; // for stopping form submit
}

function check_sim_puk_response(data) {
    console.log('- check_sim_puk_response()', data);
    var title, msg, retries, remain, msg1, msg2;
    if (data['Response'] === 'Success') {
        console.log('. check_sim_puk_response() : Success');
        $('[data-status=SignalStrength]').prop('src', '/images/transparent_1x1.png');
        go_main_page();
    } else if (data['Response'] === 'IncorrectPassword') {
        console.log('. check_sim_puk_response() : IncorrectPassword - Retries:', data['PukRetries']);
        if (data['PukRetries'] === 0) {
            show_sim_permanently_blocked_modal();
        } else {
            set_form_loading_reset();
            msg1 = 'You have entered incorrect PUK. Please contact Verizon customer support to unlock the SIM.';
            retries = data['PukRetries'];
            remain = (retries === 1) ? (retries + ' attempt remains') : (retries + ' attempts remain');
            msg2 = remain + ' until your SIM is permanently unusable.';
            change_sim_modal_msg(msg1, msg2);
            $('#PUK').focus().select();
        }
    }
}

//----------------------------------------------------------------------

function show_sim_permanently_blocked_modal() {
    console.log('- show_sim_permanently_blocked_modal()');
    // set_ok_modal() calls unbind_answer_handler();
    set_ok_modal();
    title = 'SIM is PUK Locked';
    msg = 'You have entered incorrect PUK too many times. '
        + 'The SIM is now permanently disabled. '
        + 'Please contact Verizon customer support.';
    show_modal(title, msg);
}

//----------------------------------------------------------------------

function change_sim_modal_msg(msg1, msg2) {
    $('.sim_modal_msg1').html(msg1);
    $('.sim_modal_msg2').html(msg2);
}

function set_form_loading(msg) {
    $('.sim_alert').html(msg);
    $('#OldPIN').prop('disabled', true);
    $('#PUK').prop('disabled', true);
    $('.btn-ok').prop('disabled', true);
}

function set_form_loading_reset() {
    $('.sim_alert').html('');
    $('#OldPIN').prop('disabled', false);
    $('#PUK').prop('disabled', false);
    $('.btn-ok').prop('disabled', false);
}

//----------------------------------------------------------------------

function show_no_sim_modal() {
    console.log('- show_no_sim_modal()');
    set_ok_modal();
    title = 'No SIM Found';
    msg = $('#no_sim_modal_msg').html();
    show_modal(title, msg);
}

function show_sim_error_modal() {
    console.log('- show_sim_error_modal()');
    set_ok_modal();
    title = 'SIM Error';
    msg = $('#sim_error_modal_msg').html();
    show_modal(title, msg);
}

function show_sim_error_noremains_modal() {
    console.log('- show_sim_error_noremains_modal()');
    set_ok_modal();
    title = 'SIM is locked';
    msg = $('#sim_error_noremains_modal_msg').html();
    show_modal(title, msg);
}

